// ======================================================================
//
// MeshAppear.h
// jeff grills
//
// copyright 1998 origin systems inc
//
// ======================================================================

#ifndef MESH_APPEAR_H
#define MESH_APPEAR_H

// ======================================================================
// forwards

class Iff;
class Material;

#include "Appear.h"
#include "Camera.h"
#include "Gl.def"
#include "Vector.h"

// ======================================================================

class MeshAppear : public Appear
{
	friend class GameAppear;

public:

	enum
	{
		MAX_VERTICES_PER_POLY          = USE_MAX_VERTICES_PER_POLY,
	};

protected:

	struct FaceVertex
	{
		int              vertexIndex;
		int              normalIndex;
		real             u;
		real             v;
	};

	struct Face
	{
		int               normalIndex;
		real              planeD;
		const Material   *material;
		FaceVertex       *faceVertex;
		u_int8            vertexCount;
		u_int8            flags;
		const Material   *solidMaterial;
	};

protected:

	static UVertex  vertexBuffer[MAX_VERTICES_PER_POLY];

public:

	static int             polygonSelectStart;
	static int             polygonSelectStop;

protected:

	int             vertexCount;
	Vector         *vertex;

	int             normalCount;
	Vector         *normal;

	int             faceVertexCount;
	FaceVertex     *faceVertex;

	int             faceCount;
	Face           *face;

	// bounding sphere
	Vector          center;
	real            radius;

	// mesh name
	char           *friendlyName;

	const Material **material;
	int              materialCount;

protected:

	void load_0009(const char *name, Iff &iff);
	void load_0010(const char *name, Iff &iff);
	void load_0011(const char *name, Iff &iff);
	void load_0012(const char *name, Iff &iff);
	void load_0013(const char *name, Iff &iff);

	void renderFaces(Face *face, int faceCount) const;

	void wireAxis(void) const;
	void wirePolygonNormals(void) const;
	void wireVertexNormals(void) const;
	void wireFrame(void) const;

protected:

	// disable the copy constructor and the assignment operator
	MeshAppear(void);
	MeshAppear(const MeshAppear &);
	MeshAppear &operator =(const MeshAppear &);

public:

	MeshAppear(const char *iffname, Iff &iff, real scale);
	~MeshAppear(void);

	virtual const Appear *getRenderAppear(Camera &camera, const Object *object) const;
	virtual void render(Camera &camera, const Xform &objectToWorld) const;

	void write(const char *filename) const;
};

// ======================================================================

#endif
